# Generated by Django 4.1.3 on 2022-12-18 15:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("authtoken", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="token",
            name="last_seen_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="token",
            name="scopes",
            field=models.CharField(
                blank=True,
                help_text="comma-separated list of scopes. choices are n/a.",
                max_length=50,
            ),
        ),
    ]
